<?php

namespace Ems\Controllers;

use Exception\Messages;
use Service\LogService;
use Service\OAuth\OAuthCustomImpl;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;

use Ems\Services\SolicitacaoService;
use Ems\Services\BeneficioService;
use Ems\Services\MovimentacaoService;
use Ems\Services\MinhaHistoriaService;
use Ems\Util\ExpectedException;
use Ems\Util\Utils;

class SolicitacaoController {

    /** @var OcorrenciaService */
    protected $solicitacaoService;

    protected $fieldsToDsValor = ["NRTIPOALTERA", "DTRETORNOFERIAS", "CDPAIS", "NRCURSO", "DTINIPLANEJADOTRE", "DSPESSOA", "CDFORMACOMU", "IDTPBENEFICIARIO", 
    "DTAPROVAHE", "NRESTRUTGEREN", "QTLINHATARIFA", "DSOBSBENEFICIARIO", "QTDIASFERIAS", "VRREEMBOLSO", "CDTIPOVC", "VRTARIFA_8", "QTLINHATARIFA_5", "NRTIPOLINHAVINC_6", 
    "NMPESSOAINDICADO", "QTDEDIASFERIAS", "CDBANCO", "NRMOTIVORESC", "NRBENEFICIOM", "NROCUPACAOM", "NRMOTIVOOCUPA", "NRTIPOTRANSFER", "DTINIMOVIMENT", 
    "NRTIPORELACIONA", "NRTIPOLINHAVINC", "NREVENTOM", "VREVENTOFIXO", "NRTIPOLINHAVINC_3", "NRLINHATRANSP_4", "VRTARIFA_5", "NRTIPOLINHAVINC_4", 
    "NRTIPOLINHAVINC_7", "NRTPMODALIDSAL", "NRIMOVELENDERECO", "IDFATORRHPES", "NRNACIONALIDADE", "CDTIPOENDERECO", "CDLOGRADOURO", "NMBAIRROENDERECO", 
    "NRCEPENDERECO", "NRPERFACESAPROV", "DTFIMPLANEJADOTRE", "DTHABCARTPES", "IDSEXOPESSOA", "DTINIESCALA", "NRPESSOA", "IDPUBLICAVAGA", "DTINISITUFUNC", 
    "VRTARIFA", "NRTIPOVINCULOM", "NRLINHATRANSP_5", "NRLINHATRANSP_6", "VRTARIFA_6", "NRLINHATRANSP_7", "NRPROCESSO", "DTALTESALARIO", "IDADIANT13SALAR", 
    "ABONOPECUNIARIO", "DSPRONOMETRAT", "DSCOMPLEENDERECO", "CDCOMUNICAPARC", "NRGRAUINSTR", "NRRACAPESSOA", "IDTREINAMENTOINT", "NRCARTHABPES", "CDCONTCORR", 
    "NRTURNO", "NRTPAVIPRE", "NMPROCESSO", "NRSENTIDOLINHA", "DTFIMLINHAVINC", "NRSITUFUNCM", "DTFIMSITUFUNC", "DTCANCBENEFICIARIO", "CDTIPOALTEVT", 
    "NRVINCULOM", "DTMESCOMPETENC", "VRTARIFA_2", "QTLINHATARIFA_2", "NRLINHATRANSP_3", "VRBENEFICIO", "QTLINHATARIFA_4", "NRCPFPESSOA", "NOVOVALORSALARIO", 
    "DTFIMGOZOFERIAS", "DSREFERENCIAENDE", "DSENDERECO", "NMTREINAMENTO", "NRTPDEMISSAO", "DTNASCPESSOA", "DSCATEGHABCART", "VRSALARIO", "NRTIPOOCUPACAO", 
    "NRTPMOVTRANSFM", "NRTIPOPROCESSO", "DTLIMSOLICITA", "QTINIHOR", "CDPESSOANAOGRATA", "CDPOSSUIBENSDAEMPRESA", "CDTIPOALTEVC", "QTBENEFICIO", "NMOCUPACAOH",
    "NRESTRUTLEGAL", "QTLINHATARIFA_3", "NRMOTIVOADMISSAO", "NRLINHATRANSP_8", "DTRESCISAOPROJETADA", "IDGRUPSANGUPES", "NRCONDFISPES", "CDPREFIXCOMUPARC", 
    "NRTPCONTCORR", "DTINIOCUPACAO", "NRMOTIVOTRANSF", "NRESTRUTURAM", "NRVAGAS", "NRPARCNEGOCIORESP", "NRTEMPLATE", "QTFIMHOR", "LOCALIDADE", "NRQTDIASITUFUNC", 
    "CDREPOSICAOVAGA", "DTALTELINHAVINC", "NMVINCULOM", "DTINIVIGENCIA", "NRTIPOLINHAVINC_8", "DTPREVISTAADMISSAO", "DSOBSERVACAOAUXIMORADIA", "NRMETODOTRE", 
    "DTRESCISAOVINC", "DTAVISOPREVIO", "CDAGENCIA", "NRCERTCASAPES", "DTCASAPESSOA", "NRESCALATRABM", "DTINSCBENEFICIARIO", "DSMOTIVO", "DTSOLICITA", 
    "NMPESSOADEP", "DTINILINHAVINC", "NRLINHATRANSP", "NRFERIAS", "DSGOZOFERIAS", "IDTIPOMOVIMENTO", "NRLINHATRANSP_2", "DSOBSALTEOCUPACAO", "IDTPSALARIO", "DTINIGOZOFERIAS", 
    "NMPESSOA", "CDESTACIVIL", "DSAPELIDOPESS", "SGESTADO", "CDMUNICIPIO", "NRPERFACESEXEC", "DTVALHABCARTPES", "SGUFHABCARPES", "NROCORRENCIA", "DTFIMMOVIMENT", 
    "DTNASCIFUNDPARC", "NRCPFDEP", "DSDESCRICAO", "QTDIASABONOFERI", "DSOBSLINHAVINC", "DSOBSALTESALARIO", "NRTIPOLINHAVINC_2", "VRTARIFA_3", "NRVINCULOEMPREG", 
    "VRTARIFA_4", "VRTARIFA_7", "QTLINHATARIFA_6", "QTLINHATARIFA_7", "QTLINHATARIFA_8", "NRTIPOLINHAVINC_5", "DSANOTACAO"];
    
    public function __construct(
        EntityManager $entityManager,
        SolicitacaoService $solicitacaoService,
        BeneficioService $beneficioService,
        MovimentacaoService $movimentacaoService,
        MinhaHistoriaService $minhaHistoriaService
    ) {
        $this->entityManager        = $entityManager;
        $this->solicitacaoService   = $solicitacaoService;
        $this->beneficioService     = $beneficioService;
        $this->movimentacaoService  = $movimentacaoService;
        $this->minhaHistoriaService = $minhaHistoriaService;
    }


    public function getSolicitacoesVinculosGestor(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrEstrutura            = (int) $row['NRESTRUTURAM'];
            $cdOperadorAprovante    =       $row['CDOPERADOR_APROVANTE'] ?? null;

            $solicitacoes = $this->solicitacaoService->
                getSolicitacoesVinculosGestor($nrEstrutura, $cdOperadorAprovante);

            $response->addDataSet(new DataSet('getSolicitacoesVinculosGestor', $solicitacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getSolicitacoesSelect(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrTipoSolicitacao   =       $row['NRTIPOSOLICITACAO'];
            $nrEstrutLegal       =       $row['NRESTRUTLEGAL'] ?? null;
            $nrEstrutGeren       =       $row['NRESTRUTGEREN'] ?? null;
            $nrVinculom          =       $row['NRVINCULOM']    ?? null;

            $solicitacoes = $this->solicitacaoService->
                getSolicitacoesSelect($nrTipoSolicitacao, $nrEstrutLegal, $nrEstrutGeren, $nrVinculom);

            $response->addDataSet(new DataSet('getSolicitacoesSelect', $solicitacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }
    
    public function getItemSolicitacao(Request\Row $request, Response $response) {
        try {
            $params = $request->getRow();

            $nrsolicitacao = $params['NRSOLICITACAO'];

            $solicitacoes = $this->solicitacaoService->getItemSolicitacao($nrsolicitacao);

            $response->addDataSet(new DataSet('getItemSolicitacao', $solicitacoes));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    
    public function getTiposolicitacao(Request\Row $request, Response $response) {
        try {

            $params = $request->getRow();

            $nrsolicitacao = $params['NRSOLICITACAO'] ?? null;
            $nrvinculom = $params['NRVINCULOM'] ?? null;

            $solicitacoes = $this->solicitacaoService->getTiposolicitacao($nrsolicitacao, $nrvinculom);

            $response->addDataSet(new DataSet('getTiposolicitacao', $solicitacoes));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getTipoRelacionamento(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $tiposRel = $this->solicitacaoService->getTipoRelacionamento();

            $response->addDataSet(new DataSet('getTipoRelacionamento', $tiposRel));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getLinhaTransporteAtiva(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $linhas = $this->solicitacaoService->getLinhaTransporteAtiva();

            $response->addDataSet(new DataSet('getLinhaTransporteAtiva', $linhas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTipolinhavinc(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $tiposLinhas = $this->solicitacaoService->getTipolinhavinc();

            $response->addDataSet(new DataSet('getTipolinhavinc', $tiposLinhas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getIntegtransp(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $numerosIntegracao = $this->solicitacaoService->getIntegtransp();

            $response->addDataSet(new DataSet('getIntegtransp', $numerosIntegracao));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getSentidolinha(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $sentidos = $this->solicitacaoService->getSentidolinha();

            $response->addDataSet(new DataSet('getSentidolinha', $sentidos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getBeneficios(
        Request\Row $request,
        Response    $response
    ) {
        try {

            $beneficios = $this->beneficioService->getBeneficiosComFiltro();

            $response->addDataSet(new DataSet('getBeneficios', $beneficios));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTipoBeneficio(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $tipoBeneficio = $this->beneficioService->getTipoBeneficioComFiltro();

            $response->addDataSet(new DataSet('getTipoBeneficio', $tipoBeneficio));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getPessoaRelacionamentos(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();
            $nrPessoa = (int) $row['NRPESSOA'];
            
            $pessoas = $this->solicitacaoService->getPessoaRelacionamentos($nrPessoa);

            $response->addDataSet(new DataSet('getPessoaRelacionamentos', $pessoas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTipoDependente(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $tiposDep = $this->solicitacaoService->getTipoDependente();

            $response->addDataSet(new DataSet('getTipoDependente', $tiposDep));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTemplate(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $templates = $this->solicitacaoService->getTemplate();

            $response->addDataSet(new DataSet('getTemplate', $templates));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTipoprocessos(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $tipos = $this->solicitacaoService->getTipoprocessos();

            $response->addDataSet(new DataSet('getTipoprocessos', $tipos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getParcnegocioVinculos(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $parcs = $this->solicitacaoService->getParcnegocioVinculos();

            $response->addDataSet(new DataSet('getParcnegocioVinculos', $parcs));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTipoTransferencia(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $tipos = $this->solicitacaoService->getTipoTransferencia();

            $response->addDataSet(new DataSet('getTipoTransferencia', $tipos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getMovimentacaoTransferencia(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $transferencias = $this->solicitacaoService->getMovimentacaoTransferencia();

            $response->addDataSet(new DataSet('getMovimentacaoTransferencia', $transferencias));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getMotivoTransferencia(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $motivos = $this->solicitacaoService->getMotivoTransferencia();

            $response->addDataSet(new DataSet('getMotivoTransferencia', $motivos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getTiposAnotacao(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $tiposAnotacao = $this->solicitacaoService->getTiposAnotacao();

            $response->addDataSet(new DataSet('getTiposAnotacao', $tiposAnotacao));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function aprovaVariasSolicitacoes(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $rows = $request->getRow();
            unset($rows['NRORG']);

            $resp = [
                'SUCCESS' => false,
                'ERRORS' => [],
                'ERROR_NRSOLICITACOES' => []
            ];
    
            foreach($rows as $row) {
                $nrOrg                  = (int)    $row['NRORG'];
                $nrSolicitacao          = (string) $row['NRSOLICITACAO'];
                $cdOperadorAprovante    = (string) $row['CDOPERADOR_APROVANTE'];
                $dsObservacao           = $row['DSOBSERVACAO'] ?? null;
                $extraRow               = $row['EXTRA_ROW'] ?? [];
                
                try {
                    $this->solicitacaoService->aprovaSolicitacao(
                        $nrSolicitacao,
                        $cdOperadorAprovante,
                        $dsObservacao,
                        $extraRow,
                        true
                    );
                } catch (\Exception $e) {
                    $resp['ERRORS'][] = 'Erro Nr. Solicitação - ' . $nrSolicitacao . ': ' . $e->getMessage();
                    $resp['ERROR_NRSOLICITACOES'][] = $nrSolicitacao;
                }
            }
    
            if (!empty($resp['ERRORS'])) {
                $response->addDataSet(new DataSet('aprovaVariasSolicitacoes', $resp));
            } else {
                $resp['SUCCESS'] = true;
                $response->addDataSet(new DataSet('aprovaVariasSolicitacoes', $resp));
            }

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    

    public function reprovaVariasSolicitacoes(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $rows = $request->getRow();
            unset($rows['NRORG']);

            $resp = [
                'SUCCESS' => false,
                'ERRORS' => [],
                'ERROR_NRSOLICITACOES' => []
            ];
    
            foreach($rows as $row) {
                $nrSolicitacao          = (string) $row['NRSOLICITACAO'];
                $cdOperadorAprovante    = (string) $row['CDOPERADOR_APROVANTE'];
                $dsObservacao           = $row['DSOBSERVACAO'] ?? null;
                $extraRow               = $row['EXTRA_ROW'] ?? [];
                
                try {
                    $this->solicitacaoService->reprovaSolicitacao(
                        $nrSolicitacao,
                        $cdOperadorAprovante,
                        $dsObservacao,
                        $extraRow
                    );
                } catch (\Exception $e) {
                    $resp['ERRORS'][] = 'Erro Nr. Solicitação ' . $nrSolicitacao . ': ' . $e->getMessage();
                    $resp['ERROR_NRSOLICITACOES'][] = $nrSolicitacao;
                }
            }
    
            if (!empty($resp['ERRORS'])) {
                $response->addDataSet(new DataSet('reprovaVariasSolicitacoes', $resp));
            } else {
                $resp['SUCCESS'] = true;
                $response->addDataSet(new DataSet('reprovaVariasSolicitacoes', $resp));
            }

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function aprovaSolicitacao(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrSolicitacao          = (string)  $row['NRSOLICITACAO'];
            $cdOperadorAprovante    = (string)  $row['CDOPERADOR_APROVANTE'];
            $dsObservacao           =           $row['DSOBSERVACAO']            ?? null;
            $extraRow               =           $row['EXTRA_ROW']               ?? [];
            
            $success = $this->solicitacaoService->aprovaSolicitacao(
                $nrSolicitacao,
                $cdOperadorAprovante,
                $dsObservacao,
                $extraRow
            );

            $response->addDataSet(new DataSet('aprovaSolicitacao', array($success)));
        } catch (ExpectedException $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function reprovaSolicitacao(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrSolicitacao          = (string)  $row['NRSOLICITACAO'];
            $cdOperadorAprovante    = (string)  $row['CDOPERADOR_APROVANTE'];
            $dsObservacao           =           $row['DSOBSERVACAO']            ?? null;
            $cdStatus               =           $row['CDSTATUS']                ?? null;
            $extraRow               =  (array)  ($row['EXTRA_ROW']              ?? []);
            
            $success = $this->solicitacaoService->reprovaSolicitacao(
                $nrSolicitacao,
                $cdOperadorAprovante,
                $dsObservacao,
                $extraRow
            );

            $response->addDataSet(new DataSet('reprovaSolicitacao', array($success)));
        } catch (ExpectedException $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function notificaProximosAprovadoresSolicitacao(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();
            
            $nrSolicitacao = (string) $row['NRSOLICITACAO'];
            
            $success = Utils::getInstanceByClassName(EmailService::class)->notificaProximosAprovadoresSolicitacao($nrSolicitacao);

            $response->addDataSet(new DataSet('notificaProximosAprovadoresSolicitacao', array($success)));
        } catch (ExpectedException $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function notificaOperadoresReprovacao(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();
            
            $nrSolicitacao = (string) $row['NRSOLICITACAO'];
            
            $success = Utils::getInstanceByClassName(EmailService::class)->notificaOperadoresReprovacao($nrSolicitacao);

            $response->addDataSet(new DataSet('notificaOperadoresReprovacao', array($success)));
        } catch (ExpectedException $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    
    public function getFasesSolicitacao(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrSolicitacao = (string) $row['NRSOLICITACAO'];

            $fases = $this->solicitacaoService->getFasesSolicitacao(
                $nrSolicitacao
            );

            $response->addDataSet(new DataSet('getFasesSolicitacao', $fases));
        } catch (ExpectedException $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getBeneficiosVCombust(
        Request\Row $request,
        Response    $response
    ) {
        try {

            $beneficios = $this->beneficioService->getBeneficiosVCombust();

            $response->addDataSet(new DataSet('getBeneficiosVCombust', $beneficios));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getBeneficiosVManutencao(
        Request\Row $request,
        Response    $response
    ) {
        try {

            $beneficios = $this->beneficioService->getBeneficiosVManutencao();

            $response->addDataSet(new DataSet('getBeneficiosVManutencao', $beneficios));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getDadosPreviosSolicitacao(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo           = (int) $row['NRVINCULOM'];
            $nrTipoSolicitacao   =  (int) $row['NRTIPOSOLICITACAO'];

            $solicitacaoMethods = [
                1  => 'getDadosFeriasVinculo',
                2  => 'getAlteracaoTransferencia',
                9  => 'getAlteracaoOcupacao',
                14 => 'getAlteracaoEscalaTrabalho',
                10 => 'getLinhasVinculosVT'
            ];

            $solicitacoes = [];
    
            if (array_key_exists($nrTipoSolicitacao, $solicitacaoMethods)) {
                $methodName = $solicitacaoMethods[$nrTipoSolicitacao];
                $solicitacoes = $this->solicitacaoService->$methodName($nrVinculo);
            } else {
                throw new \Exception('Tipo de solicitação inválido');
            }

            $response->addDataSet(new DataSet('getDadosPreviosSolicitacao', $solicitacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function verificaTipoTransferencia(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrtipotransf     = (int) $row['NRTIPOTRANSFER'];
            $nrTipoEstrutura  =  (int) $row['NRTIPOESTRUTURA'];
            
            $this->movimentacaoService->verificaTipoTransferencia($nrtipotransf, $nrTipoEstrutura);
            
            $dataset = [
                'success' => true
            ];

            $response->addDataSet(new DataSet('verificaTipoTransferencia', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getSolicitacaoSubstituicaoOrVinculoSubst(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo     = (int) $row['NRVINCULOM'];

            $solicitacao = $this->solicitacaoService->getSolicitacaoSubstituicaoOrVinculoSubst($nrVinculo);

            $response->addDataSet(new DataSet('getSolicitacaoSubstituicaoOrVinculoSubst', $solicitacao));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getDadosVinculo(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo     = (int) $row['NRVINCULOM'];

            $solicitacao = $this->solicitacaoService->getDadosVinculo($nrVinculo);

            $response->addDataSet(new DataSet('getDadosVinculo', $solicitacao));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function setDataSoliSubstituicaoVinculo(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo     = (int) $row['NRVINCULOM'];
            $nrSolicitacao = (int) $row['NRSOLICITACAO'];

            $this->solicitacaoService->setDataSoliSubstituicaoVinculo($nrVinculo, $nrSolicitacao);

            $dataset = [
                'success' => true
            ];

            $response->addDataSet(new DataSet('setDataSoliSubstituicaoVinculo', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getFolhaFechada(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculom  = (string) $row['NRVINCULOM'];
            $dtAlteracao = (string) $row['DTALTERACAO'];

            $folha = $this->solicitacaoService->getFolhaFechada($nrVinculom, $dtAlteracao);

            $response->addDataSet(new DataSet('getFolhaFechada', $folha));
        } catch (ExpectedException $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getHierarquiaByEstruturaLegalAndGeren(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrEstrutLegal = (string) $row['NRESTRUTLEGAL'];
            $nrEstrutGeren  = (string) $row['NRESTRUTGEREN'];

            $hierarquia = $this->solicitacaoService->getHierarquiaByEstruturaLegalAndGeren($nrEstrutLegal, $nrEstrutGeren);

            $response->addDataSet(new DataSet('getHierarquiaByEstruturaLegalAndGeren', $hierarquia));
        } catch (ExpectedException $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
    public function getSolicitacoesAdmissao(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $solicitacao = $this->solicitacaoService->getSolicitacoesAdmissao();

            $response->addDataSet(new DataSet('getSolicitacoesAdmissao', $solicitacao));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function editSolicitacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            $nrorg = $row['NRORG'];
            $nrSolicitacao = $row['NRSOLICITACAO'];

            $this->minhaHistoriaService->editSolicitacao($row);

            // Anexo ao email
            $filesPath = $this->minhaHistoriaService->lidarComFiles($row);

            $this->minhaHistoriaService->setDocumentosSolicitacao($row);


            $this->solicitacaoService->resetFases($nrSolicitacao, $nrorg);
            
            $dataset = [
                'success' => true
            ];

            $response->addDataSet(new DataSet('editSolicitacao', $dataset));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getMotivoRecusa(Request\Row $request, Response $response) {
        try {

            $params = $request->getRow();

            $nrsolicitacao = $params['NRSOLICITACAO'];

            $motivoRecusas = $this->solicitacaoService->getMotivoRecusa($nrsolicitacao);
            $motivos = [];

            if(!empty($motivoRecusas)) {
                foreach ($motivoRecusas as $motivoRecusa) {
                    $motivo = 'Solicitação reprovada na fase: "' . $motivoRecusa['NMFASEAPROVSOLI'] . '" pelo operador: "' . $motivoRecusa['CDOPERINCLUSAO'] . '-' . $motivoRecusa['NMOPERADOR'] . '" no dia: ' . $motivoRecusa['DTINCLUSAO'] . ' pelo motivo: "' . $motivoRecusa['DSOBSERVACAO'] . '"';
                    $motivos[] = $motivo;
                }
            }
            
            $response->addDataSet(new DataSet('getMotivoRecusa', $motivos));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getTodosVinculosRescindidos(Request\Row $request, Response $response) {
        try {

            $vinculos = $this->solicitacaoService->getTodosVinculosRescindidos();

            $response->addDataSet(new DataSet('getTodosVinculosRescindidos', $vinculos));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function generateSolicitacaoReport(Request\Row $request, Response $response)
    {
        try {
            $params = $request->getRow();
            $nrtiposolicitacao = $params['NRTIPOSOLICITACAO'];

            $filters = json_decode($params['FILTERS'], true);
            
            $file = $this->solicitacaoService->generateSolicitacaoReport($filters, $nrtiposolicitacao);

            $response->addDataSet(new DataSet('generateSolicitacaoReport', $file));
    
        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }
    

    public function getColumnsSolicitacaoReport(Request\Row $request, Response $response) {
        try {

            $params = $request->getRow();

            $nrtiposolicitacao = $params['NRTIPOSOLICITACAO'];

            $columns = $this->solicitacaoService->getColumnsSolicitacaoReport($nrtiposolicitacao);
            
            $response->addDataSet(new DataSet('getColumnsSolicitacaoReport', $columns));

        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function getSolicitacoesComItemsReportXLS(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrorg = $row['NRORG'];

            $nrTipoSolicitacao = $row['NRTIPOSOLICITACAO'];

            $filters = json_decode($row['FILTERS'], true);

            $dataset = $this->solicitacaoService->getSolicitacoesComItemsReportXLS($filters, $nrTipoSolicitacao);
            
            $response->addDataSet(new DataSet('getSolicitacoesComItemsReportXLS', $dataset));
        } catch (\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        }
    }

    public function cancelaSolicitacaoDesligamento(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();
            $nrorg                  = (string)  $row['NRORG'];
            $nrSolicitacao          = (string)  $row['NRSOLICITACAO'];
            $nrVinculom             = (string)  $row['NRVINCULOM'];
            $cdOperadorAprovante    = (string)  $row['CDOPERADOR_APROVANTE'];
            $dsObservacao           =           $row['DSOBSERVACAO']            ?? null;
            $extraRow               =           $row['EXTRA_ROW']               ?? [];
            $nrPeriodo              = (string) $row['NRPERIODO']['NRPERIODOAPURACAO'] ?? null;
            
            $success = $this->solicitacaoService->cancelaSolicitacaoDesligamento(
                $nrSolicitacao,
                $cdOperadorAprovante,
                $dsObservacao,
                $extraRow,
                $nrorg,
                $nrVinculom,
                $nrPeriodo
            );

            $response->addDataSet(new DataSet('cancelaSolicitacaoDesligamento', array($success)));
        } catch (ExpectedException $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
}
