<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;
use Util\{ NovoCodigo };

class SolicitacaoReportRepository {
    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->connection = $entityManager->getConnection();
    }

    
    // Default
    public function getSolicitacaoReportDefault($params, $itemsQuery) {
        $sql = <<<SQL
            SELECT
                SL.NRORG,
                SL.NRSOLICITACAO,
                SL.NRTIPOSOLICITACAO,
                SL.NRVINCULOM,
                TO_CHAR(SL.DTSOLICITACAO,'DD/MM/YYYY') AS DTSOLICITACAO,
                DECODE (SL.IDAPROVACAOSEC,  'A', 'Aprovada',
                                            'R', 'Reprovada',
                                            'P', 'Pendente',
                                            'E', 'Em andamento',
                                            'Pendente') DSAPROVACAOSEC,
                SL.DTAPROVACAOPRI,
                SL.CDOPERAPROVPRI,
                SL.IDAPROVACAOSEC,
                SL.DTAPROVACAOSEC,
                SL.CDOPERAPROVSEC,
                SL.DSOBSSOLICITACAO,
                SL.DSOBSAPROVAPRI,
                SL.DSOBSAPROVASEC,
                VM_S.NMVINCULOM,
                EML.NRESTRUTURAM || ' | ' || EML.NMESTRUTURAM AS NMESTRUTLEGAL,
                EMG.NRESTRUTURAM || ' | ' || EMG.NMESTRUTURAM AS NMESTRUTGEREN,
                TP.NMTIPOSOLICITACAO,
                OP.NMOPERADOR,
                FASE_SEGUINTE.NRFASEAPROVSOLI,
                FASE_SEGUINTE.NRORDEM || '-' || FASE_SEGUINTE.NMFASEAPROVSOLI AS NMFASEAPROVSOLI,
                FASE_SEGUINTE_PROXIMA.NRFASEAPROVSOLI AS NRFASEAPROVSOLI_SEGUINTE,
                FASE_SEGUINTE_PROXIMA.NRORDEM || '-' || FASE_SEGUINTE_PROXIMA.NMFASEAPROVSOLI AS NMFASEAPROVSOLI_SEGUINTE,
                AE.NRESCALATRABM || ' | ' || ETH.NMESCALATRABH AS HISTORICO_NRESCALATRABM,
                AE.NRTURNO || ' | ' || TN.DSTURNO AS HISTORICO_NRTURNO,
                EHL.NRESTRUTURAM || ' | ' ||  EHL.NMESTRUTURAH AS HISTORICO_NRESTRUTLEGAL,
                EHG.NRESTRUTURAM || ' | ' ||  EHG.NMESTRUTURAH AS HISTORICO_NRESTRUTGEREN,
                CASE 
                    WHEN ATS.VRSALARIO IS NULL THEN 'R$ 0,00'
                    ELSE 'R$ ' || TRIM(TO_CHAR(NVL(ATS.VRSALARIO, 0), '999G999G999D99', 'NLS_NUMERIC_CHARACTERS = '',.'''))
                END AS HISTORICO_VRSALARIO,
                ATS.IDTPSALARIO AS HISTORICO_IDTPSALARIO,
                OCUPH.NROCUPACAOM || ' | ' || OCUPH.NMOCUPACAOH AS HISTORICO_NROCUPACAOM
            FROM
                GPE_SOLICITACAO SL
                LEFT JOIN
                GPE_TIPOSOLICITACAO TP
                    ON SL.NRTIPOSOLICITACAO = TP.NRTIPOSOLICITACAO
                LEFT JOIN
                GPE_FASETRAMSOLI FT
                    ON FT.NRORG = SL.NRORG 
                        AND FT.IDATIVO = 'S'
                        AND FT.NRSOLICITACAO = SL.NRSOLICITACAO
                        AND FT.NRFASETRAMSOLI = (
                                SELECT
                                    MAX(FT1.NRFASETRAMSOLI)
                                FROM
                                    GPE_FASETRAMSOLI FT1
                                WHERE
                                    FT1.NRORG = FT.NRORG AND
                                    FT1.NRSOLICITACAO = FT.NRSOLICITACAO AND
                                    FT1.IDATIVO = 'S'
                            )
                LEFT JOIN
                GPE_FASEAPROVSOLI FASE_ATUAL
                    ON FASE_ATUAL.NRORG = FT.NRORG
                    AND FASE_ATUAL.NRFASEAPROVSOLI = FT.NRFASEAPROVSOLI
                LEFT JOIN
                GPE_FASEAPROVSOLI FASE_SEGUINTE
                    ON FASE_SEGUINTE.NRORG = SL.NRORG
                    AND FASE_SEGUINTE.NRMODELOAPROVSOLI = SL.NRMODELOAPROVSOLI
                    AND FASE_SEGUINTE.NRORDEM = NVL(FASE_ATUAL.NRORDEM, 0) + 1
                LEFT JOIN
                GPE_FASEAPROVSOLI FASE_SEGUINTE_PROXIMA
                    ON FASE_SEGUINTE_PROXIMA.NRORG = FASE_SEGUINTE.NRORG
                    AND FASE_SEGUINTE_PROXIMA.NRMODELOAPROVSOLI = SL.NRMODELOAPROVSOLI
                    AND FASE_SEGUINTE_PROXIMA.NRORDEM = NVL(FASE_SEGUINTE.NRORDEM, 0) + 1
                LEFT JOIN
                GPE_VINCULOM VM_S
                    ON VM_S.NRORG = SL.NRORG
                        AND VM_S.NRVINCULOM = SL.NRVINCULOM
                LEFT JOIN
                GPE_VINCULOH VH_S
                    ON VH_S.NRORG = VM_S.NRORG
                        AND VH_S.NRVINCULOM = VM_S.NRVINCULOM
                        AND VH_S.DTMESCOMPETENC =
                            (SELECT DTMESCOMPETENC FROM 
                                (SELECT 
                                    V1.DTMESCOMPETENC 
                                FROM 
                                    GPE_VINCULOH V1 
                                WHERE 
                                    TO_CHAR(V1.DTMESCOMPETENC,'DD/MM/YYYY') <= TO_CHAR(SL.DTSOLICITACAO,'DD/MM/YYYY')
                                    AND V1.NRORG = VH_S.NRORG 
                                    AND V1.NRVINCULOM = VH_S.NRVINCULOM 
                                    ORDER BY V1.DTMESCOMPETENC DESC) 
                            WHERE ROWNUM = 1)
                LEFT JOIN OPERADOR OP
                    ON OP.CDOPERADOR = SL.CDOPERINCLUSAO
                LEFT JOIN PTO_PERIODOAPURACAO PA
                    ON PA.NRORG = SL.NRORG
                    AND PA.NRPERIODOAPURACAO = :P_NRPERIODOAPURACAO
                LEFT JOIN ESTRUTURAM EML
                    ON EML.NRESTRUTURAM = VH_S.NRESTRUTLEGAL AND EML.NRORG = SL.NRORG
                LEFT JOIN ESTRUTURAM EMG
                    ON EMG.NRESTRUTURAM = VH_S.NRESTRUTGEREN AND EMG.NRORG = SL.NRORG
                
                -- Escala
                LEFT JOIN GPE_ALTEESCALA AE 
                    ON AE.NRORG = SL.NRORG 
                    AND AE.NRVINCULOM = VM_S.NRVINCULOM
                    AND AE.DTINIESCALA = (
                        SELECT MAX(AE2.DTINIESCALA) 
                        FROM GPE_ALTEESCALA AE2
                        WHERE AE2.NRORG = AE.NRORG
                        AND AE2.NRVINCULOM = AE.NRVINCULOM
                        AND AE2.DTINIESCALA <= SL.DTSOLICITACAO
                    )
                LEFT JOIN GPE_ESCALATRABH ETH 
                    ON ETH.NRORG = SL.NRORG 
                    AND ETH.NRESCALATRABM = AE.NRESCALATRABM
                    AND ETH.DTMESCOMPETENC = (
                        SELECT MAX(ETHH.DTMESCOMPETENC)
                        FROM GPE_ESCALATRABH ETHH
                        WHERE ETHH.NRORG = ETH.NRORG
                        AND ETHH.NRESCALATRABM = ETH.NRESCALATRABM
                        AND ETHH.DTMESCOMPETENC <= SL.DTSOLICITACAO
                    )
                LEFT JOIN GPE_TURNO TN 
                    ON TN.NRORG = SL.NRORG
                    AND TN.NRTURNO = AE.NRTURNO
                    AND TN.NRESCALATRABM = AE.NRESCALATRABM
                    AND TN.IDATIVO = 'S'
                
                -- Estrutura Legal
                LEFT JOIN ESTRUTURAH EHL ON
                    EHL.NRORG = SL.NRORG AND
                    EHL.NRESTRUTURAM = VH_S.NRESTRUTLEGAL AND
                    EHL.DTMESCOMPETENC = (
                        SELECT MAX(EHL2.DTMESCOMPETENC)
                        FROM ESTRUTURAH EHL2
                        WHERE EHL2.NRORG = EHL.NRORG
                        AND EHL2.NRESTRUTURAM = EHL.NRESTRUTURAM
                        AND EHL2.DTMESCOMPETENC <= SL.DTSOLICITACAO
                    )

                -- Estrutura Gerencial
                LEFT JOIN ESTRUTURAH EHG ON
                    EHG.NRORG = SL.NRORG AND
                    EHG.NRESTRUTURAM = VH_S.NRESTRUTGEREN AND
                    EHG.DTMESCOMPETENC = (
                        SELECT MAX(EHG2.DTMESCOMPETENC)
                        FROM ESTRUTURAH EHG2
                        WHERE EHG2.NRORG = EHG.NRORG
                        AND EHG2.NRESTRUTURAM = EHG.NRESTRUTURAM
                        AND EHG2.DTMESCOMPETENC <= SL.DTSOLICITACAO
                    )

                -- Salario
                LEFT JOIN GPE_ALTESALARIO ATS ON
                    ATS.NRORG = SL.NRORG AND
                    ATS.NRVINCULOM = VH_S.NRVINCULOM AND
                    ATS.DTALTESALARIO = (
                        SELECT MAX(ATS2.DTALTESALARIO)
                        FROM GPE_ALTESALARIO ATS2
                        WHERE ATS2.NRORG = ATS.NRORG
                        AND ATS2.NRVINCULOM = ATS.NRVINCULOM
                        AND ATS2.DTALTESALARIO <= SL.DTSOLICITACAO
                    )

                -- Ocupação      
                LEFT JOIN GPE_OCUPACAOH OCUPH ON
                    OCUPH.NROCUPACAOM = VH_S.NRCARGO AND
                    OCUPH.NRORG = SL.NRORG AND
                    OCUPH.DTMESCOMPETENC = (
                        SELECT MAX(OCUPH2.DTMESCOMPETENC)
                        FROM GPE_OCUPACAOH OCUPH2
                        WHERE OCUPH2.NRORG = OCUPH.NRORG
                        AND OCUPH2.NROCUPACAOM = OCUPH.NROCUPACAOM
                        AND OCUPH2.DTMESCOMPETENC <= SL.DTSOLICITACAO
                    )
                
            WHERE
                (SL.NRORG = :P_NRORG OR SL.NRORG = 0)
                AND SL.NRTIPOSOLICITACAO = :P_NRTIPOSOLICITACAO
                AND (:P_NRSOLICITACAO IS NULL OR SL.NRSOLICITACAO = :P_NRSOLICITACAO)
                AND (:P_NRVINCULOM IS NULL OR SL.NRVINCULOM = :P_NRVINCULOM)
                AND (:P_DSAPROVACAOSEC IS NULL OR SL.IDAPROVACAOSEC = :P_DSAPROVACAOSEC)
                AND (:P_NRESTRUTLEGAL IS NOT NULL AND VH_S.NRESTRUTLEGAL = :P_NRESTRUTLEGAL)
                AND (:P_NRESTRUTGEREN IS NULL OR VH_S.NRESTRUTGEREN = :P_NRESTRUTGEREN)
                $itemsQuery
            ORDER BY SL.NRSOLICITACAO ASC
SQL;
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $solicitacoes = $std->fetchAll();

        return $solicitacoes;
    }
    
    // FERIAS
    public function getSolicitacaoReportFerias($params) {
        $sql = <<<SQL
            SELECT
                SL.NRORG,
                SL.NRSOLICITACAO,
                SL.NRTIPOSOLICITACAO,
                SL.NRVINCULOM,
                TO_CHAR(SL.DTSOLICITACAO,'DD/MM/YYYY') AS DTSOLICITACAO,
                DECODE (SL.IDAPROVACAOSEC,  'A', 'Aprovada',
                                            'R', 'Reprovada',
                                            'P', 'Pendente',
                                            'E', 'Em andamento',
                                            'Pendente') DSAPROVACAOSEC,
                SL.DTAPROVACAOPRI,
                SL.CDOPERAPROVPRI,
                SL.IDAPROVACAOSEC,
                SL.DTAPROVACAOSEC,
                SL.CDOPERAPROVSEC,
                SL.DSOBSSOLICITACAO,
                SL.DSOBSAPROVAPRI,
                SL.DSOBSAPROVASEC,
                VM_S.NMVINCULOM,
                EML.NMESTRUTURAM AS NMESTRUTLEGAL,
                EMG.NMESTRUTURAM AS NMESTRUTGEREN,
                TP.NMTIPOSOLICITACAO,
                OP.NMOPERADOR,
                FASE_SEGUINTE.NRFASEAPROVSOLI,
                FASE_SEGUINTE.NRORDEM || '-' || FASE_SEGUINTE.NMFASEAPROVSOLI AS NMFASEAPROVSOLI,
                FASE_SEGUINTE_PROXIMA.NRFASEAPROVSOLI AS NRFASEAPROVSOLI_SEGUINTE,
                FASE_SEGUINTE_PROXIMA.NRORDEM || '-' || FASE_SEGUINTE_PROXIMA.NMFASEAPROVSOLI AS NMFASEAPROVSOLI_SEGUINTE,
                ETH.NMESCALATRABH
            FROM
                GPE_SOLICITACAO SL
                LEFT JOIN
                GPE_TIPOSOLICITACAO TP
                    ON SL.NRTIPOSOLICITACAO = TP.NRTIPOSOLICITACAO
                LEFT JOIN
                GPE_FASETRAMSOLI FT
                    ON FT.NRORG = SL.NRORG 
                        AND FT.IDATIVO = 'S'
                        AND FT.NRSOLICITACAO = SL.NRSOLICITACAO
                        AND FT.NRFASETRAMSOLI = (
                                SELECT
                                    MAX(FT1.NRFASETRAMSOLI)
                                FROM
                                    GPE_FASETRAMSOLI FT1
                                WHERE
                                    FT1.NRORG = FT.NRORG AND
                                    FT1.NRSOLICITACAO = FT.NRSOLICITACAO AND
                                    FT1.IDATIVO = 'S'
                            )
                LEFT JOIN
                GPE_FASEAPROVSOLI FASE_ATUAL
                    ON FASE_ATUAL.NRORG = FT.NRORG
                    AND FASE_ATUAL.NRFASEAPROVSOLI = FT.NRFASEAPROVSOLI
                LEFT JOIN
                GPE_FASEAPROVSOLI FASE_SEGUINTE
                    ON FASE_SEGUINTE.NRORG = SL.NRORG
                    AND FASE_SEGUINTE.NRMODELOAPROVSOLI = SL.NRMODELOAPROVSOLI
                    AND FASE_SEGUINTE.NRORDEM = NVL(FASE_ATUAL.NRORDEM, 0) + 1
                LEFT JOIN
                GPE_FASEAPROVSOLI FASE_SEGUINTE_PROXIMA
                    ON FASE_SEGUINTE_PROXIMA.NRORG = FASE_SEGUINTE.NRORG
                    AND FASE_SEGUINTE_PROXIMA.NRMODELOAPROVSOLI = SL.NRMODELOAPROVSOLI
                    AND FASE_SEGUINTE_PROXIMA.NRORDEM = NVL(FASE_SEGUINTE.NRORDEM, 0) + 1
                LEFT JOIN
                GPE_VINCULOM VM_S
                    ON VM_S.NRORG = SL.NRORG
                        AND VM_S.NRVINCULOM = SL.NRVINCULOM
                LEFT JOIN
                GPE_VINCULOH VH_S
                    ON VH_S.NRORG = VM_S.NRORG
                        AND VH_S.NRVINCULOM = VM_S.NRVINCULOM
                        AND VH_S.DTMESCOMPETENC =
                            (SELECT DTMESCOMPETENC FROM 
                                (SELECT 
                                    V1.DTMESCOMPETENC 
                                FROM 
                                    GPE_VINCULOH V1 
                                WHERE 
                                    TO_CHAR(V1.DTMESCOMPETENC,'DD/MM/YYYY') <= TO_CHAR(SL.DTSOLICITACAO,'DD/MM/YYYY')
                                    AND V1.NRORG = VH_S.NRORG 
                                    AND V1.NRVINCULOM = VH_S.NRVINCULOM 
                                    ORDER BY V1.DTMESCOMPETENC DESC) 
                            WHERE ROWNUM = 1)
                LEFT JOIN OPERADOR OP
                    ON OP.CDOPERADOR = SL.CDOPERINCLUSAO
                LEFT JOIN PTO_PERIODOAPURACAO PA
                    ON PA.NRORG = SL.NRORG
                    AND PA.NRPERIODOAPURACAO = :P_NRPERIODOAPURACAO
                LEFT JOIN ESTRUTURAM EML
                    ON EML.NRESTRUTURAM = VH_S.NRESTRUTLEGAL AND EML.NRORG = SL.NRORG
                LEFT JOIN ESTRUTURAM EMG
                    ON EMG.NRESTRUTURAM = VH_S.NRESTRUTGEREN AND EMG.NRORG = SL.NRORG
                LEFT JOIN GPE_ESCALATRABH ETH 
                    ON ETH.NRORG = VM_S.NRORG 
                    AND ETH.NRESCALATRABM = VH_S.NRESCALATRABM 
                    AND ETH.DTMESCOMPETENC = (
                        SELECT MAX(ETHH.DTMESCOMPETENC)
                        FROM GPE_ESCALATRABH ETHH
                        WHERE ETHH.NRORG = ETH.NRORG
                        AND ETHH.NRESCALATRABM = ETH.NRESCALATRABM
                        AND TO_CHAR(ETHH.DTMESCOMPETENC,'DD/MM/YYYY') <= TO_CHAR(SL.DTSOLICITACAO, 'DD/MM/YYYY')
                    )
            WHERE
                (SL.NRORG = :P_NRORG OR SL.NRORG = 0)
                AND SL.NRTIPOSOLICITACAO = :P_NRTIPOSOLICITACAO
                AND (:P_DSAPROVACAOSEC IS NULL OR SL.IDAPROVACAOSEC = :P_DSAPROVACAOSEC)
                AND (:P_TIPOLEGAL IS NOT NULL AND VH_S.NRESTRUTLEGAL = :P_TIPOLEGAL)
                AND (:P_NRESTRUTURAS IS NULL OR VH_S.NRESTRUTGEREN = :P_NRESTRUTURAS)
            ORDER BY SL.NRSOLICITACAO ASC
SQL;
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $solicitacoes = $std->fetchAll();

        return $solicitacoes;
    }

    
}